==> bec_widgets-0.68.0.tar.gz: bec_widgets-0.68.0/bec_widgets/widgets/bec_status_box/status_item.py <==
""" Module for a StatusItem widget to display status and metrics for a BEC service.
The widget is bound to be used with the BECStatusBox widget."""

import enum
import sys
from datetime import datetime

import qdarktheme
from bec_lib.utils.import_utils import lazy_import_from
from pydantic import Field
from qtpy.QtCore import Qt, Slot
from qtpy.QtWidgets import QDialog, QHBoxLayout, QLabel, QStyle, QVBoxLayout, QWidget

from bec_widgets.utils.bec_connector import ConnectionConfig

# TODO : Put normal imports back when Pydantic gets faster
BECStatus = lazy_import_from("bec_lib.messages", ("BECStatus",))


class IconsEnum(enum.Enum):
    """Enum class for icons in the status item widget."""

    RUNNING = "SP_DialogApplyButton"
    BUSY = "SP_BrowserReload"
    IDLE = "SP_MessageBoxWarning"
    ERROR = "SP_DialogCancelButton"
    NOTCONNECTED = "SP_TitleBarContextHelpButton"


class StatusWidgetConfig(ConnectionConfig):
    """Configuration class for the status item widget."""

    service_name: str
    status: str
    info: dict
    metrics: dict | None
    icon_size: tuple = Field(default=(24, 24), description="The size of the icon in the widget.")
    font_size: int = Field(16, description="The font size of the text in the widget.")


class StatusItem(QWidget):
    """A widget to display the status of a service.

    Args:
        parent: The parent widget.
        config (dict): The configuration for the service.
    """

    def __init__(self, parent=None, config: dict = None):
        if config is None:
            config = StatusWidgetConfig(widget_class=self.__class__.__name__)
        else:
            if isinstance(config, dict):
                config = StatusWidgetConfig(**config)
            self.config = config
        QWidget.__init__(self, parent=parent)
        self.parent = parent
        self.layout = None
        self.config = config
        self._popup_label_ref = {}
        self._label = None
        self._icon = None
        self.init_ui()

    def init_ui(self) -> None:
        """Init the UI for the status item widget."""
        self.layout = QHBoxLayout()
        self.layout.setContentsMargins(5, 5, 5, 5)
        self.setLayout(self.layout)
        self._label = QLabel()
        self._icon = QLabel()
        self.layout.addWidget(self._label)
        self.layout.addWidget(self._icon)
        self.update_ui()

    @Slot(dict)
    def update_config(self, config: dict) -> None:
        """Update the configuration of the status item widget.
        This method is invoked from the parent widget.
        The UI values are later updated based on the new configuration.

        Args:
            config (dict): Config updates from parent widget.
        """
        if config["service_name"] != self.config.service_name:
            return
        self.config.status = config["status"]
        self.config.info = config["info"]
        self.config.metrics = config["metrics"]
        self.update_ui()

    def update_ui(self) -> None:
        """Update the UI of the labels, and popup dialog."""
        self.set_text()
        self.set_status()
        self._set_popup_text()

    def set_text(self) -> None:
        """Set the text of the QLabel basae on the config."""
        service = self.config.service_name
        status = self.config.status
        if "BECClient" in service.split("/"):
            service = service.split("/")[0] + "/..." + service.split("/")[1][-4:]
        if status == "NOTCONNECTED":
            status = "NOT CONNECTED"
        text = f"{service} is {status}"
        self._label.setText(text)

    def set_status(self) -> None:
        """Set the status icon for the status item widget."""
        icon_name = IconsEnum[self.config.status].value
        icon = self.style().standardIcon(getattr(QStyle.StandardPixmap, icon_name))
        self._icon.setPixmap(icon.pixmap(*self.config.icon_size))
        self._icon.setAlignment(Qt.AlignmentFlag.AlignRight)

    def show_popup(self) -> None:
        """Method that is invoked when the user double clicks on the StatusItem widget."""
        dialog = QDialog(self)
        dialog.setWindowTitle(f"{self.config.service_name} Details")
        layout = QVBoxLayout()
        popup_label = self._make_popup_label()
        self._set_popup_text()
        layout.addWidget(popup_label)
        dialog.setLayout(layout)
        dialog.finished.connect(self._cleanup_popup_label)
        dialog.exec()

    def _make_popup_label(self) -> QLabel:
        """Create a QLabel for the popup dialog.

        Returns:
            QLabel: The label for the popup dialog.
        """
        label = QLabel()
        label.setWordWrap(True)
        self._popup_label_ref.update({"label": label})
        return label

    def _set_popup_text(self) -> None:
        """Compile the metrics text for the status item widget."""
        if self._popup_label_ref.get("label") is None:
            return
        metrics_text = (
            f"<b>SERVICE:</b> {self.config.service_name}<br><b>STATUS:</b> {self.config.status}<br>"
        )
        if self.config.metrics:
            for key, value in self.config.metrics.items():
                if key == "create_time":
                    value = datetime.fromtimestamp(value).strftime("%Y-%m-%d %H:%M:%S")
                metrics_text += f"<b>{key.upper()}:</b> {value}<br>"
        self._popup_label_ref["label"].setText(metrics_text)

    def _cleanup_popup_label(self) -> None:
        """Cleanup the popup label."""
        self._popup_label_ref.clear()


def main():
    """Run the status item widget."""
    # pylint: disable=import-outside-toplevel
    from qtpy.QtWidgets import QApplication

    app = QApplication(sys.argv)
    qdarktheme.setup_theme("auto")
    main_window = StatusItem()
    main_window.show()
    sys.exit(app.exec())


if __name__ == "__main__":
    main()

==> bec_widgets-0.90.0.tar.gz: bec_widgets-0.90.0/bec_widgets/widgets/bec_status_box/status_item.py <==
""" Module for a StatusItem widget to display status and metrics for a BEC service.
The widget is bound to be used with the BECStatusBox widget."""

import enum
from datetime import datetime

from bec_lib.utils.import_utils import lazy_import_from
from qtpy.QtCore import Qt, Slot
from qtpy.QtWidgets import QDialog, QHBoxLayout, QLabel, QStyle, QVBoxLayout, QWidget

# TODO : Put normal imports back when Pydantic gets faster
BECStatus = lazy_import_from("bec_lib.messages", ("BECStatus",))


class IconsEnum(enum.Enum):
    """Enum class for icons in the status item widget."""

    RUNNING = "SP_DialogApplyButton"
    BUSY = "SP_BrowserReload"
    IDLE = "SP_MessageBoxWarning"
    ERROR = "SP_DialogCancelButton"
    NOTCONNECTED = "SP_TitleBarContextHelpButton"


class StatusItem(QWidget):
    """A widget to display the status of a service.

    Args:
        parent: The parent widget.
        config (dict): The configuration for the service, must be a BECServiceInfoContainer.
    """

    def __init__(self, parent: QWidget = None, config=None):
        QWidget.__init__(self, parent=parent)
        if config is None:
            # needed because we need parent to be the first argument for QT Designer
            raise ValueError(
                "Please initialize the StatusItem with a BECServiceInfoContainer for config, received None."
            )
        self.config = config
        self.parent = parent
        self.layout = None
        self._label = None
        self._icon = None
        self.icon_size = (24, 24)

        self._popup_label_ref = {}
        self.init_ui()

    def init_ui(self) -> None:
        """Init the UI for the status item widget."""
        self.layout = QHBoxLayout()
        self.layout.setContentsMargins(5, 5, 5, 5)
        self.setLayout(self.layout)
        self._label = QLabel()
        self._icon = QLabel()
        self.layout.addWidget(self._label)
        self.layout.addWidget(self._icon)
        self.update_ui()

    @Slot(dict)
    def update_config(self, config) -> None:
        """Update the config of the status item widget.

        Args:
            config (dict): Config updates from parent widget, must be a BECServiceInfoContainer.
        """
        if self.config is None or config.service_name != self.config.service_name:
            return
        self.config = config
        self.update_ui()

    def update_ui(self) -> None:
        """Update the UI of the labels, and popup dialog."""
        if self.config is None:
            return
        self.set_text()
        self.set_status()
        self._set_popup_text()

    def set_text(self) -> None:
        """Set the text of the QLabel basae on the config."""
        service = self.config.service_name
        status = self.config.status
        if len(service.split("/")) > 1 and service.split("/")[0].startswith("BEC"):
            service = service.split("/", maxsplit=1)[0] + "/..." + service.split("/")[1][-4:]
        if status == "NOTCONNECTED":
            status = "NOT CONNECTED"
        text = f"{service} is {status}"
        self._label.setText(text)

    def set_status(self) -> None:
        """Set the status icon for the status item widget."""
        icon_name = IconsEnum[self.config.status].value
        icon = self.style().standardIcon(getattr(QStyle.StandardPixmap, icon_name))
        self._icon.setPixmap(icon.pixmap(*self.icon_size))
        self._icon.setAlignment(Qt.AlignmentFlag.AlignRight)

    def show_popup(self) -> None:
        """Method that is invoked when the user double clicks on the StatusItem widget."""
        dialog = QDialog(self)
        dialog.setWindowTitle(f"{self.config.service_name} Details")
        layout = QVBoxLayout()
        popup_label = self._make_popup_label()
        self._set_popup_text()
        layout.addWidget(popup_label)
        dialog.setLayout(layout)
        dialog.finished.connect(self._cleanup_popup_label)
        dialog.exec()

    def _make_popup_label(self) -> QLabel:
        """Create a QLabel for the popup dialog.

        Returns:
            QLabel: The label for the popup dialog.
        """
        label = QLabel()
        label.setWordWrap(True)
        self._popup_label_ref.update({"label": label})
        return label

    def _set_popup_text(self) -> None:
        """Compile the metrics text for the status item widget."""
        if self._popup_label_ref.get("label") is None:
            return
        metrics_text = (
            f"<b>SERVICE:</b> {self.config.service_name}<br><b>STATUS:</b> {self.config.status}<br>"
        )
        if self.config.metrics:
            for key, value in self.config.metrics.items():
                if key == "create_time":
                    value = datetime.fromtimestamp(value).strftime("%Y-%m-%d %H:%M:%S")
                metrics_text += f"<b>{key.upper()}:</b> {value}<br>"
        self._popup_label_ref["label"].setText(metrics_text)

    def _cleanup_popup_label(self) -> None:
        """Cleanup the popup label."""
        self._popup_label_ref.clear()
